'use strict';

var view = require('@codemirror/view');
var language = require('@codemirror/language');
var highlight = require('@lezer/highlight');
var _utils = require('@utils');

/**
 * Enhanced Monokai theme color definitions
 * -------------------------------------
 * Colors organized by function with visual color blocks
 */
// Base colors
const base00 = '#272822'; // Background
const base01 = '#f8f8f2'; // Foreground
// const base02 = '#4a4a76'; // Selection
const base03 = '#88846f'; // Comments, invisibles
const base04 = '#f8f8f0'; // Cursor
// Accent colors
const base05 = '#F92672'; // Keyword, Storage, Tag - Pink
const base06 = '#FD971F'; // Variable, Parameter - Orange
const base07 = '#66D9EF'; // Function, Type - Blue
const base08 = '#E6DB74'; // String, RegExp - Yellow
const base09 = '#AE81FF'; // Constant, Number - Purple
const base0A = '#A6E22E'; // Class, Heading - Green
// UI specific colors
const invalid = '#F44747'; // Error color - Red
const darkBackground = '#414339'; // Gutter background
const highlightBackground = '#3e3d3257'; // Line highlight with opacity
const tooltipBackground = '#34352f'; // Tooltip background - Slightly lighter than base00
const selection = '#49483E'; // Selection background
const selectionMatch = '#75715e70'; // Selection match with opacity
const cursor = base04; // Cursor color
const activeBracketBg = '#75715E55'; // Active bracket background with opacity
const activeBracketBorder = base07; // Active bracket border - blue
const diagnosticWarning = base06; // Warning color - orange
const linkColor = base07; // Link color - blue
const visitedLinkColor = base09; // Visited link color - purple
/**
 * Enhanced editor theme styles for Monokai
 */
const monokaiTheme = view.EditorView.theme({
    // Base editor styles
    '&': {
        color: base01,
        backgroundColor: base00,
        fontSize: _utils.generalContent.fontSize,
        fontFamily: _utils.generalContent.fontFamily,
    },
    // Content and cursor
    '.cm-content': {
        caretColor: cursor,
        lineHeight: _utils.generalContent.lineHeight,
    },
    '.cm-cursor, .cm-dropCursor': {
        borderLeftColor: cursor,
        borderLeftWidth: _utils.generalCursor.borderLeftWidth,
    },
    '.cm-fat-cursor': {
        backgroundColor: `${cursor}99`,
        color: base00,
    },
    // Selection
    '&.cm-focused > .cm-scroller > .cm-selectionLayer .cm-selectionBackground, .cm-selectionBackground, .cm-content ::selection': {
        backgroundColor: selection,
    },
    // Make sure selection appears above active line
    '.cm-selectionLayer': {
        zIndex: 100,
    },
    // Search functionality
    '.cm-searchMatch': {
        backgroundColor: '#49483E99',
        outline: `1px solid ${base08}`,
        color: base01,
        borderRadius: _utils.generalSearchField.borderRadius,
        '& span': {
            color: base01,
        },
    },
    '.cm-searchMatch.cm-searchMatch-selected': {
        backgroundColor: base05,
        color: base00,
        padding: _utils.generalSearchField.padding,
        '& span': {
            color: base00,
        },
    },
    '.cm-search.cm-panel.cm-textfield': {
        color: base01,
        borderRadius: _utils.generalSearchField.borderRadius,
        padding: _utils.generalSearchField.padding,
    },
    // Panels
    '.cm-panels': {
        backgroundColor: darkBackground,
        color: base01,
        borderRadius: '4px',
    },
    '.cm-panels.cm-panels-top': {
        borderBottom: `1px solid ${base03}80`,
    },
    '.cm-panels.cm-panels-bottom': {
        borderTop: `1px solid ${base03}80`,
    },
    '.cm-panel button': {
        backgroundColor: tooltipBackground,
        color: base01,
        border: _utils.generalPanel.border,
        borderRadius: _utils.generalPanel.borderRadius,
        padding: _utils.generalPanel.padding,
    },
    '.cm-panel button:hover': {
        backgroundColor: '#49483E',
    },
    // Line highlighting
    '.cm-activeLine': {
        backgroundColor: highlightBackground,
        borderRadius: _utils.generalLine.borderRadius,
        zIndex: 1,
    },
    // Gutters
    '.cm-gutters': {
        backgroundColor: darkBackground,
        color: base03,
        border: _utils.generalGutter.border,
        borderRight: '1px solid #3e3d32',
        paddingRight: _utils.generalGutter.paddingRight,
    },
    '.cm-activeLineGutter': {
        backgroundColor: '#35352c',
        color: base01,
        fontWeight: _utils.generalGutter.fontWeight,
    },
    '.cm-lineNumbers': {
        fontSize: _utils.generalGutter.fontSize,
    },
    '.cm-foldGutter': {
        fontSize: _utils.generalGutter.fontSize,
    },
    '.cm-foldGutter .cm-gutterElement': {
        color: base03,
        cursor: 'pointer',
    },
    '.cm-foldGutter .cm-gutterElement:hover': {
        color: base01,
    },
    // Tooltips and autocomplete
    '.cm-tooltip': {
        backgroundColor: tooltipBackground,
        border: '1px solid #49483E',
        borderRadius: _utils.generalTooltip.borderRadius,
        padding: _utils.generalTooltip.padding,
        boxShadow: '0 2px 8px rgba(0, 0, 0, 0.4)',
    },
    '.cm-tooltip-autocomplete': {
        '& > ul': {
            backgroundColor: tooltipBackground,
            border: 'none',
        },
        '& > ul > li': {
            padding: _utils.generalTooltip.padding,
            lineHeight: _utils.generalTooltip.lineHeight,
        },
        '& > ul > li[aria-selected]': {
            backgroundColor: selection,
            color: base01,
            borderRadius: _utils.generalTooltip.borderRadiusSelected,
        },
        '& > ul > li > span.cm-completionIcon': {
            color: base03,
            paddingRight: _utils.generalTooltip.paddingRight,
        },
        '& > ul > li > span.cm-completionDetail': {
            color: base03,
            fontStyle: 'italic',
        },
    },
    '.cm-tooltip .cm-tooltip-arrow:before': {
        borderTopColor: 'transparent',
        borderBottomColor: 'transparent',
    },
    '.cm-tooltip .cm-tooltip-arrow:after': {
        borderTopColor: tooltipBackground,
        borderBottomColor: tooltipBackground,
    },
    // Diagnostics styling
    '.cm-diagnostic': {
        '&-error': {
            borderLeft: `3px solid ${invalid}`,
        },
        '&-warning': {
            borderLeft: `3px solid ${diagnosticWarning}`,
        },
        '&-info': {
            borderLeft: `3px solid ${linkColor}`,
        },
    },
    '.cm-lintPoint-error': {
        borderBottom: `2px wavy ${invalid}`,
    },
    '.cm-lintPoint-warning': {
        borderBottom: `2px wavy ${diagnosticWarning}`,
    },
    // Matching brackets
    '.cm-matchingBracket': {
        backgroundColor: activeBracketBg,
        outline: `1px solid ${activeBracketBorder}`,
        borderRadius: _utils.generalMatching.borderRadius,
    },
    '.cm-nonmatchingBracket': {
        backgroundColor: '#f9267240',
        outline: `1px solid ${invalid}`,
        borderRadius: _utils.generalMatching.borderRadius,
    },
    // Selection matches
    '.cm-selectionMatch': {
        backgroundColor: selectionMatch,
        outline: `1px solid ${base03}50`,
        borderRadius: _utils.generalMatching.borderRadius,
    },
    // Fold placeholder
    '.cm-foldPlaceholder': {
        backgroundColor: tooltipBackground,
        color: base03,
        border: `1px dotted ${base03}70`,
        borderRadius: _utils.generalPlaceholder.borderRadius,
        padding: _utils.generalPlaceholder.padding,
        margin: _utils.generalPlaceholder.margin,
    },
    // Focus outline
    '&.cm-focused': {
        outline: 'none',
        boxShadow: `0 0 0 2px ${base00}, 0 0 0 3px ${base07}40`,
    },
    // Scrollbars
    '& .cm-scroller::-webkit-scrollbar': {
        width: _utils.generalScroller.width,
        height: _utils.generalScroller.height,
    },
    '& .cm-scroller::-webkit-scrollbar-track': {
        background: darkBackground,
    },
    '& .cm-scroller::-webkit-scrollbar-thumb': {
        backgroundColor: '#49483E',
        borderRadius: _utils.generalScroller.borderRadius,
        border: `3px solid ${darkBackground}`,
    },
    '& .cm-scroller::-webkit-scrollbar-thumb:hover': {
        backgroundColor: '#5a594d',
    },
    // Ghost text
    '.cm-ghostText': {
        opacity: '0.5',
        color: base03,
    },
}, { dark: true });
/**
 * Enhanced syntax highlighting for Monokai theme
 */
const monokaiHighlightStyle = language.HighlightStyle.define([
    // Keywords and control flow
    { tag: highlight.tags.keyword, color: base05, fontWeight: 'bold' },
    { tag: highlight.tags.controlKeyword, color: base05, fontWeight: 'bold' },
    { tag: highlight.tags.moduleKeyword, color: base05, fontWeight: 'bold' },
    // Names and variables
    { tag: [highlight.tags.name, highlight.tags.deleted, highlight.tags.character, highlight.tags.macroName], color: base06 },
    { tag: [highlight.tags.variableName], color: base06 },
    { tag: [highlight.tags.propertyName], color: base0A, fontStyle: 'normal' },
    // Classes and types
    { tag: [highlight.tags.typeName], color: base07, fontStyle: 'italic' },
    { tag: [highlight.tags.className], color: base0A, fontStyle: 'italic' },
    { tag: [highlight.tags.namespace], color: base06, fontStyle: 'italic' },
    // Operators and punctuation
    { tag: [highlight.tags.operator, highlight.tags.operatorKeyword], color: base05 },
    { tag: [highlight.tags.bracket], color: base01 },
    { tag: [highlight.tags.brace], color: base01 },
    { tag: [highlight.tags.punctuation], color: base01 },
    // Functions and parameters
    { tag: [highlight.tags.function(highlight.tags.variableName), highlight.tags.labelName], color: base07 },
    { tag: [highlight.tags.definition(highlight.tags.function(highlight.tags.variableName))], color: base07 },
    { tag: [highlight.tags.definition(highlight.tags.variableName)], color: base06 },
    // Constants and literals
    { tag: highlight.tags.number, color: base09 },
    { tag: highlight.tags.changed, color: base09 },
    { tag: highlight.tags.annotation, color: invalid, fontStyle: 'italic' },
    { tag: highlight.tags.modifier, color: base09, fontStyle: 'italic' },
    { tag: highlight.tags.self, color: base09 },
    { tag: [highlight.tags.color, highlight.tags.constant(highlight.tags.name), highlight.tags.standard(highlight.tags.name)], color: base09 },
    { tag: [highlight.tags.atom, highlight.tags.bool, highlight.tags.special(highlight.tags.variableName)], color: base09 },
    // Strings and regex
    { tag: [highlight.tags.processingInstruction, highlight.tags.inserted], color: base0A },
    { tag: [highlight.tags.special(highlight.tags.string), highlight.tags.regexp], color: base08 },
    { tag: highlight.tags.string, color: base08 },
    // Punctuation and structure
    { tag: highlight.tags.definition(highlight.tags.typeName), color: base07, fontWeight: 'bold' },
    // Comments and documentation
    { tag: highlight.tags.meta, color: base03 },
    { tag: highlight.tags.comment, fontStyle: 'italic', color: base03 },
    { tag: highlight.tags.docComment, fontStyle: 'italic', color: base03 },
    // HTML/XML elements
    { tag: [highlight.tags.tagName], color: base05 },
    { tag: [highlight.tags.attributeName], color: base0A },
    // Markdown and text formatting
    { tag: [highlight.tags.heading], fontWeight: 'bold', color: base0A },
    { tag: [highlight.tags.strong], fontWeight: 'bold', color: base06 },
    { tag: [highlight.tags.emphasis], fontStyle: 'italic', color: base06 },
    // Links and URLs
    {
        tag: [highlight.tags.link],
        color: visitedLinkColor,
        fontWeight: '500',
        textDecoration: 'underline',
        textUnderlinePosition: 'under',
    },
    {
        tag: [highlight.tags.url],
        color: linkColor,
        textDecoration: 'underline',
        textUnderlineOffset: '2px',
    },
    // Special states
    {
        tag: [highlight.tags.invalid],
        color: base01,
        textDecoration: 'underline wavy',
        borderBottom: `1px wavy ${invalid}`,
    },
    { tag: [highlight.tags.strikethrough], color: invalid, textDecoration: 'line-through' },
    // Enhanced syntax highlighting
    { tag: highlight.tags.constant(highlight.tags.name), color: base09 },
    { tag: highlight.tags.deleted, color: invalid },
    { tag: highlight.tags.squareBracket, color: base01 },
    { tag: highlight.tags.angleBracket, color: base01 },
    // Additional specific styles
    { tag: highlight.tags.monospace, color: base01 },
    { tag: [highlight.tags.contentSeparator], color: base06 },
    { tag: highlight.tags.quote, color: base03 },
]);
/**
 * Combined Monokai theme extension
 */
const monokai = [
    monokaiTheme,
    language.syntaxHighlighting(monokaiHighlightStyle),
];

exports.monokai = monokai;
exports.monokaiHighlightStyle = monokaiHighlightStyle;
exports.monokaiTheme = monokaiTheme;
